from . import TwitAnalyzer
from . import TwitStream
from time import sleep
from progress.spinner import *
from prettytable import *
from termcolor import cprint, colored

'''
Module for processing live twitter data
'''
class TwitLive:
    def __init__(self, analyzer=None):
        if analyzer == None:
            self.analyzer = TwitAnalyzer()
        else:
            self.analyzer = analyzer

    # Create and start a Twitter stream
    def stream(self, query, live):
        twit_stream = TwitStream(self.analyzer.config['CONSUMER_KEY'],self.analyzer.config['CONSUMER_SECRET'],self.analyzer.config['ACCESS_TOKEN'],self.analyzer.config['ACCESS_TOKEN_SECRET'], live=live)
        thread = twit_stream.filter(track=[query], stall_warnings=True, threaded=True)
        return twit_stream, thread

    # Display progress spinner for certain amount of seconds
    def progress(self, text, secs):
        spin = PixelSpinner(text)
        for i in range(secs*4):
            spin.next()
            sleep(.25)
        spin.finish()

    # Process live twitter trend data
    # 
    # location (str) - location for which to analyze trends from
    # num_trends (int) - number of trends to analyze
    # display (bool) - boolean to indicate whether or not to display live stream output to the console
    def TrendAnalysis(self, location, num_trends, display):
        trends = self.analyzer.get_trends(self.analyzer.trend_locations[location]["woeid"])
        data={}
        total_tweets = 0
        total_reg_tweets = 0
        total_retweets = 0
        total_unique_retweets = 0
        total_volume = 0

        if num_trends == 'all':
            num_trends = len(trends)

        print(f"Gathering data on top {num_trends} trends from [ {location} ]")
        for i, trend in enumerate(trends[:num_trends]):
            # Start stream and print status
            streem, thread = self.stream(trend['name'], display)
            if not display:
                self.progress(f" {i+1}/{num_trends} [ {colored(trend['name'],'magenta')} ] - Volume: {trend['tweet_volume']:,} ", 30)
            else:
                print(f" {i+1}/{num_trends} [ {trend['name']} ] - Volume: {trend['tweet_volume']:,}")
                sleep(30)
            

            # Disconnect stream and wait for thread to finish
            streem.disconnect()
            thread.join()

            total_tweets += streem.tweets
            total_reg_tweets += streem.reg_tweets
            total_retweets += streem.retweets
            total_unique_retweets += streem.get_unique_retweets()
            total_volume += trend['tweet_volume']
            
            data[trend['name']] = { 'tweets':streem.tweets,
                                    'reg_tweets':streem.reg_tweets,
                                    'retweets':streem.retweets, 
                                    'unique_retweets':streem.get_unique_retweets(),
                                    'perc_retweets':streem.get_perc_retweets(),
                                    'perc_unique_retweets':streem.get_perc_unique_retweets(),
                                    'sentiment':( round(streem.pos/streem.tweets*100,2), round(streem.neg/streem.tweets*100,2) ),
                                    'tw_p_min': streem.tweets*2}


        # Create results table
        table = PrettyTable(['Trend', 'Total Tweets', 'Sentiment % (+/-)', 'Regular Tweets', 'Retweets', 'Unique Retweets', 'twt/min', '% Retweets', '% Unique Retweets'])
        table.set_style(SINGLE_BORDER)
        table.align = 'l'

        for trend in data:
            table.add_row([trend, data[trend]['tweets'], data[trend]['sentiment'], data[trend]['reg_tweets'], data[trend]['retweets'], data[trend]['unique_retweets'], data[trend]['tw_p_min'], data[trend]['perc_retweets'], data[trend]['perc_unique_retweets']])
        table.add_row(['Summary', total_tweets, '', total_reg_tweets, total_retweets, total_unique_retweets, round(total_tweets/(num_trends/2)), round((total_retweets/total_tweets)*100,2), round((total_unique_retweets/total_retweets)* 100,2)])    

        print("\n")
        print(f"Summary of top {num_trends} trends from [ {colored(location,'magenta')} ]")
        print(table)
        print(f"\nProcessed {round((total_tweets/total_volume)*100,4)}% of total volume - [ {total_tweets:,} tweets ]")
        print(f"[{total_reg_tweets} regular ] [ {total_retweets} retweets ] [ {total_unique_retweets} unique retweets ]")
        print("\n")



    # Process live twitter search data
    def SearchAnalysis(self):
        pass



import tweepy
from termcolor import cprint, colored
from textblob import TextBlob

'''
# Custom stream class for streaming live tweet data
# 
# live=True -> streamed tweets will be printed to the console
'''
class TwitStream(tweepy.Stream):
    def __init__(self, consumer_key, consumer_secret, acces_token, access_token_secret, live=True):
        super().__init__(consumer_key, consumer_secret, acces_token, access_token_secret)
        self.tweets = 0
        self.reg_tweets = 0
        self.retweets = 0
        self.is_live = live
        self.unique_retweets = []
        self.pos = 0
        self.neg = 0
        self.impact_raw = 0

    def get_perc_retweets(self):
        return round((self.retweets/self.tweets)*100,2)

    def get_perc_unique_retweets(self):
        if self.retweets == 0:
            return 0
        return round((self.get_unique_retweets()/self.retweets)*100,2)

    def get_unique_retweets(self):
        return len(self.unique_retweets)

    # Gets the sum of followers of the tweet's author as well as any users who retweeted
    def get_impact_raw(self, tweet):
        followers = tweet.author.followers_count
        for retweet in tweet.retweets():
            followers += retweet.author.followers_count

        self.impact_raw += followers

    def print_tweet(self, tweet, text, quoted_text, url, quote_url):
        # Print header for tweet or retweet
        if hasattr(tweet, 'retweeted_status'):
            print(f"{colored(tweet.author.name,'cyan')} retweeted {colored(tweet.retweeted_status.author.name,'yellow')}")

        else:
            print(f"{colored(tweet.author.name,'cyan')} tweeted")

        # Print location if there
        if tweet.author.location:
            print(f"[{colored(tweet.author.location,'magenta')}] --- [{colored(url,'red')}]")
        else:
            print(f"[{colored('***','magenta')}] --- [{colored(url,'red')}]")


        # Print tweet contents
        print(text)

        # Print quote if there
        if hasattr(tweet, 'quoted_status'):
            print(f"\n=====\n{colored(tweet.quoted_status.author.name,'yellow')} - [{colored(quote_url,'red')}]\n{colored(quoted_text,'blue')}\n=====")

        print("---\n\n")

    # Get the url associated with the given tweet
    def get_url(self, tweet):
        return f"https://twitter.com/twitter/statuses/{tweet.id}"

    # Get text associated with the given tweet
    def get_text(self, status):
        if hasattr(status, 'full_text'):
            return status.full_text
        else:
            return status.text
            
    # Get sentiment of tweet
    # TODO: update to account for varying levels of polarity
    def get_sentiment(self, tweet):
        blob = TextBlob(self.get_text(tweet))
        if blob.polarity > 0:
            self.pos += 1
        else:
            self.neg += 1

    def on_closed(self, response):
        pass

    def on_status(self, status):
        retweet_text = ""
        retweet_url = ""
        quoted_text = ""
        quote_url = ""

        self.get_sentiment(status)
        self.get_followers(status)
        
        # Get text from retweet
        if hasattr(status, 'retweeted_status'):
            retweet_url = self.get_url(status.retweeted_status)
            retweet_text = self.get_text(status.retweeted_status)
            if status.retweeted_status.id not in self.unique_retweets:
                self.unique_retweets.append(status.retweeted_status.id)

            self.retweets += 1
        else:
            self.reg_tweets += 1
        # Get text from quoted tweet
        if hasattr(status, 'quoted_status'):
            quote_url = self.get_url(status.quoted_status)
            quoted_text = self.get_text(status.quoted_status)

        # Get text from tweet
        url = self.get_url(status)
        text = self.get_text(status)

        if self.is_live:
            if hasattr(status, 'retweeted_status'):
                self.print_tweet(status, retweet_text, quoted_text, retweet_url, quote_url)
            else:
                self.print_tweet(status, text, quoted_text, url, quote_url)

        self.tweets += 1


import tweepy
import yaml
import os
from requests.utils import unquote
from textblob import TextBlob

'''
# Class for processing and analyzing Tweets
'''
class TwitAnalyzer:
    def __init__(self):
        self.config = None
        self.api = self.init_twitter()
        self.trend_locations = self.get_trend_locations()

    # Calculate sample size to ensure accuracy
    # default margin of error = 3%
    # default confidence interval 95% (1.96)
    def sample_size(self, pop, z=1.96, err=.03):
        numerator = (z**2 * .25) / err**2
        denominator = 1 + (z**2 * .25) / (err**2 * pop)
        return round(numerator/denominator,2)

    # Initialize configuration and twitter API connection
    def init_twitter(self):
        if not os.path.isfile('.config'):
            print("[ ERROR ]: Could not find '.config' file in the current directory")
        with open('.config') as file:
            self.config = yaml.load(file, Loader=yaml.FullLoader)
            
        # Initialize twitter connection
        auth = tweepy.OAuthHandler(self.config['CONSUMER_KEY'],self.config['CONSUMER_SECRET'])
        auth.set_access_token(self.config['ACCESS_TOKEN'],self.config['ACCESS_TOKEN_SECRET'])
         
        api = tweepy.API(auth,wait_on_rate_limit=True)
        return api 

    # Get map of available trend locations
    def get_trend_locations(self):
        trend_locations = {}
        trends = self.api.available_trends()

        for trend in trends:
            trend_locations[trend['name']] = {  'woeid': trend['woeid'], 
                                                'parent': trend['parentid']}

        return trend_locations

    # Get trends from given location
    def get_trends(self, woeid):
        trend_info = []
        trends = self.api.get_place_trends(woeid)[0]
        trend_date = trends['created_at']
        for trend in trends['trends']:
            trend_info.append(trend)
            # if trend['tweet_volume'] and trend not in trend_info:
            #     trend_info.append(trend)

        return sorted(trend_info, key=lambda trend: trend['tweet_volume'] if trend['tweet_volume'] is not None else -1, reverse=True)


    # Check if tweet is a retweet
    def is_retweet(self, tweet):
        return tweet.retweeted 

    # Get tweet id
    def tweet_id(self, tweet):
        return tweet.id 

    # Get favorite count of tweet
    def favorite_count(self, tweet):
        return tweet.favorite_count

    # Get number of times this tweet has been retweeted
    def retweet_count(self, tweet):
        return tweet.retweet_count

    # Get tweet source url if it exists
    def tweet_url(self, tweet):
        return f"https://twitter.com/twitter/statuses/{tweet.id}"
    
    # Get location of tweets author if it exists
    def tweet_location(self, tweet):
        if len(tweet.author.location) > 0:
            return tweet.author.location
        return None

    # Get author's follower count
    def follower_count(self, tweet):
        return tweet.author.followers_count

    # Get the url associated with the given tweet
    def get_url(self, tweet):
        return f"https://twitter.com/twitter/statuses/{tweet.id}"

    # Get text associated with the given tweet
    def get_text(self, status):
        if hasattr(status, 'full_text'):
            return status.full_text
        else:
            return status.text

    # Get sentiment of tweet
    # TODO: update to account for sentiment levels? Tuning of the actual sentiment gathered?
    def get_sentiment(self, tweet):
        blob = TextBlob(self.get_text(tweet))
        return blob.polarity

    # Estimate the number of people reached by this tweet
    def get_impact_raw(self, tweet):
        return self.get_followers(tweet)

    # Gets the sum of followers of the tweet's author as well as any users who retweeted
    def get_followers(self, tweet):
        followers = tweet.author.followers_count
        for retweet in tweet.retweets():
            followers += retweet.author.followers_count

        return followers

    # Scrape tweets related to specified topic 
    # NOTE: THIS FILTERS RETWEETS
    # Returns - List of tweet objects
    def get_topic_data(self, topic, max_tweets):
        results = self.api.search_tweets(q=f"{topic} -filter:retweets", result_type='recent',tweet_mode='extended', count=100)
        data = list(results)
        max_id = results.max_id

        while len(data) <= max_tweets:
            print(len(data))
            results = self.api.search_tweets(q=f"{topic} -filter:retweets", result_type='recent',tweet_mode='extended', count=100, max_id=max_id)
            if len(results) == 0:
                print(f"Ran out of Tweets matching search . . . [ {len(data)} ]")
                break
            data += list(results)
            max_id = results.max_id-1

        return data




from . import TwitAnalyzer
from . import TwitStream

'''
Module for processing twitter data offline
'''
class TwitProcess:
    def __init__(self):
        self.analyzer = TwitAnalyzer()
    

    # Process bulk twitter data related to specified query
    def BulkAnalysis(self, query):
        tweets = analyzer.api.search_tweets(query,count=100)
        for tweet in tweets:
            pass
            # TODO: Sentiment Processing 

            # TODO: Save query data to db
            # db.hset(f"topic:{query}","tweets",tweet.text)
            # db.hset(f"topic:{query}","likes",tweet.favorite_count)
            # db.expire(f"topic:{query}", TIMEOUT)
            


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="TwitAnalysis",
    version="1.0.19",
    author="Michael Mondoro",
    author_email="michaelmondoro@gmail.com",
    description="Package for analyzing Twitter data",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/michaelMondoro/TwitAnalysis",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "tweepy",
        "PyYAML",
        "termcolor",
        "progress",
        "prettytable",
        "textblob",
    ],
    python_requires='>=3.10',
)
